/**
 * NetXMS - open source network management system
 * Copyright (C) 2003-2021 Victor Kirhenshtein
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.netxms.ui.eclipse.nxsl.views;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.commands.ActionHandler;
import org.eclipse.ui.contexts.IContextService;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.part.ViewPart;
import org.netxms.client.NXCSession;
import org.netxms.client.Script;
import org.netxms.ui.eclipse.actions.RefreshAction;
import org.netxms.ui.eclipse.console.resources.SharedIcons;
import org.netxms.ui.eclipse.jobs.ConsoleJob;
import org.netxms.ui.eclipse.nxsl.Activator;
import org.netxms.ui.eclipse.nxsl.Messages;
import org.netxms.ui.eclipse.nxsl.dialogs.CreateScriptDialog;
import org.netxms.ui.eclipse.nxsl.views.helpers.ScriptComparator;
import org.netxms.ui.eclipse.nxsl.views.helpers.ScriptFilter;
import org.netxms.ui.eclipse.nxsl.views.helpers.ScriptLabelProvider;
import org.netxms.ui.eclipse.shared.ConsoleSharedData;
import org.netxms.ui.eclipse.tools.MessageDialogHelper;
import org.netxms.ui.eclipse.tools.WidgetHelper;
import org.netxms.ui.eclipse.widgets.FilterText;
import org.netxms.ui.eclipse.widgets.SortableTableViewer;

/**
 * Script library view
 */
@SuppressWarnings("deprecation")
public class ScriptLibrary extends ViewPart
{
	public static final String ID = "org.netxms.ui.eclipse.nxsl.views.ScriptLibrary"; //$NON-NLS-1$
	
	// Columns
	public static final int COLUMN_ID = 0;
	public static final int COLUMN_NAME = 1;

	private static final String TABLE_CONFIG_PREFIX = "ScriptLibrary"; //$NON-NLS-1$
	
	private NXCSession session;
	private SortableTableViewer viewer;
	private RefreshAction actionRefresh;
	private Action actionNew;
	private Action actionEdit;
	private Action actionRename;
	private Action actionDelete;
	private Action actionCopyName;
   private Action actionShowFilter;
   
   private ScriptFilter filter;
   private FilterText filterText;
   private IDialogSettings settings;
   private Composite content;

     
   /* (non-Javadoc)
    * @see org.eclipse.ui.part.ViewPart#init(org.eclipse.ui.IViewSite)
    */
   @Override
   public void init(IViewSite site) throws PartInitException
   {
      super.init(site);
      session = ConsoleSharedData.getSession();
      settings = Activator.getDefault().getDialogSettings();
   }

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent)
	{		
      content = new Composite(parent, SWT.NONE);
      content.setLayout(new FormLayout());
      
      // Create filter area
      filterText = new FilterText(content, SWT.NONE);
      filterText.addModifyListener(new ModifyListener() {
         @Override
         public void modifyText(ModifyEvent e)
         {
            onFilterModify();
         }
      });
		
		final String[] names = { Messages.get().ScriptLibrary_ColumnId, Messages.get().ScriptLibrary_ColumnName };
		final int[] widths = { 70, 400 };
		viewer = new SortableTableViewer(content, names, widths, 0, SWT.UP, SortableTableViewer.DEFAULT_STYLE);
		WidgetHelper.restoreTableViewerSettings(viewer, Activator.getDefault().getDialogSettings(), TABLE_CONFIG_PREFIX);
		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setLabelProvider(new ScriptLabelProvider());
		viewer.setComparator(new ScriptComparator());
		filter = new ScriptFilter();
		viewer.addFilter(filter);
		viewer.addSelectionChangedListener(new ISelectionChangedListener() {
			@Override
			public void selectionChanged(SelectionChangedEvent event)
			{
				IStructuredSelection selection = (IStructuredSelection)event.getSelection();
				if (selection != null)
				{
					actionEdit.setEnabled(selection.size() == 1);
					actionRename.setEnabled(selection.size() == 1);
					actionDelete.setEnabled(selection.size() > 0);
					actionCopyName.setEnabled(selection.size() == 1);
				}
			}
		});
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			@Override
			public void doubleClick(DoubleClickEvent event)
			{
				actionEdit.run();
			}
		});
		viewer.getTable().addDisposeListener(new DisposeListener() {
			@Override
			public void widgetDisposed(DisposeEvent e)
			{
				WidgetHelper.saveTableViewerSettings(viewer, Activator.getDefault().getDialogSettings(), TABLE_CONFIG_PREFIX);
			}
		});
      
      // Setup layout
      FormData fd = new FormData();
      fd.left = new FormAttachment(0, 0);
      fd.top = new FormAttachment(filterText);
      fd.right = new FormAttachment(100, 0);
      fd.bottom = new FormAttachment(100, 0);
      viewer.getTable().setLayoutData(fd);
      
      fd = new FormData();
      fd.left = new FormAttachment(0, 0);
      fd.top = new FormAttachment(0, 0);
      fd.right = new FormAttachment(100, 0);
      filterText.setLayoutData(fd);

      filterText.setCloseAction(new Action() {
         @Override
         public void run()
         {
            enableFilter(false);
            actionShowFilter.setChecked(false);
         }
      });

      activateContext();
      createActions();
      contributeToActionBars();
      createPopupMenu();

      // Set initial focus to filter input line
      if (actionShowFilter.isChecked())
         filterText.setFocus();
      else
         enableFilter(false); // Will hide filter area correctly

		refreshScriptList();
	}
	
   /**
    * Activate context
    */
   private void activateContext()
   {
      IContextService contextService = (IContextService)getSite().getService(IContextService.class);
      if (contextService != null)
      {
         contextService.activateContext("org.netxms.ui.eclipse.nxsl.context.ScriptLibrary"); //$NON-NLS-1$
      }
   }

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus()
	{
		viewer.getTable().setFocus();
	}
	
	/**
	 * Create actions
	 */
   private void createActions()
	{
	   final IHandlerService handlerService = (IHandlerService)getSite().getService(IHandlerService.class);
	   
		actionRefresh = new RefreshAction() {
			@Override
			public void run()
			{
				refreshScriptList();
			}
		};

		actionNew = new Action(Messages.get().ScriptLibrary_New, Activator.getImageDescriptor("icons/new.png")) { //$NON-NLS-1$
			@Override
			public void run()
			{
				createNewScript();
			}
		};

		actionEdit = new Action(Messages.get().ScriptLibrary_Edit, Activator.getImageDescriptor("icons/edit.png")) { //$NON-NLS-1$
			@Override
			public void run()
			{
				editScript();
			}
		};
		actionEdit.setEnabled(false);

		actionRename = new Action(Messages.get().ScriptLibrary_Rename) {
			@Override
			public void run()
			{
				renameScript();
			}
		};
		actionRename.setEnabled(false);

		actionDelete = new Action(Messages.get().ScriptLibrary_Delete, Activator.getImageDescriptor("icons/delete.png")) { //$NON-NLS-1$
			@Override
			public void run()
			{
				deleteScript();
			}
		};
		actionDelete.setEnabled(false);
		
		actionCopyName = new Action("Copy name", SharedIcons.COPY) {
		   @Override
		   public void run()
		   {
		      copyNameToClipboard();
		   }
		};
		actionCopyName.setActionDefinitionId("org.netxms.ui.eclipse.nxsl.commands.CopyNameToClipboard");
		handlerService.activateHandler(actionCopyName.getActionDefinitionId(), new ActionHandler(actionCopyName));
      
      actionShowFilter = new Action("Show filter", Action.AS_CHECK_BOX) {
         @Override
         public void run()
         {
            enableFilter(actionShowFilter.isChecked());
         }
      };
      actionShowFilter.setImageDescriptor(SharedIcons.FILTER);
      actionShowFilter.setChecked(getBooleanFromSettings("ScriptLibrary.showFilter", true));
      actionShowFilter.setActionDefinitionId("org.netxms.ui.eclipse.datacollection.commands.show_dci_filter"); //$NON-NLS-1$
      handlerService.activateHandler(actionShowFilter.getActionDefinitionId(), new ActionHandler(actionShowFilter));
	}

	/**
	 * Contribute actions to action bar
	 */
	private void contributeToActionBars()
	{
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	/**
	 * Fill local pull-down menu
	 * 
	 * @param manager
	 *           Menu manager for pull-down menu
	 */
	private void fillLocalPullDown(IMenuManager manager)
	{
		manager.add(actionNew);
		manager.add(actionDelete);
		manager.add(actionRename);
		manager.add(actionEdit);
      manager.add(new Separator());
      manager.add(actionShowFilter);
		manager.add(new Separator());
		manager.add(actionRefresh);
	}

	/**
	 * Fill local tool bar
	 * 
	 * @param manager
	 *           Menu manager for local toolbar
	 */
	private void fillLocalToolBar(IToolBarManager manager)
	{
		manager.add(actionNew);
		manager.add(actionEdit);
		manager.add(actionDelete);
      manager.add(new Separator());
      manager.add(actionShowFilter);
		manager.add(new Separator());
		manager.add(actionRefresh);
	}
	
	/**
	 * Create pop-up menu
	 */
	private void createPopupMenu()
	{
		// Create menu manager.
		MenuManager menuMgr = new MenuManager();
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager mgr)
			{
				fillContextMenu(mgr);
			}
		});

		// Create menu.
		Menu menu = menuMgr.createContextMenu(viewer.getTable());
		viewer.getTable().setMenu(menu);
	}

	/**
	 * Fill context menu
	 * 
	 * @param mgr Menu manager
	 */
	protected void fillContextMenu(final IMenuManager mgr)
	{
		mgr.add(actionNew);
		mgr.add(actionEdit);
		mgr.add(actionRename);
		mgr.add(actionDelete);
		mgr.add(actionCopyName);
	}

	/**
	 * Reload script list from server
	 */
	private void refreshScriptList()
	{
		new ConsoleJob(Messages.get().ScriptLibrary_LoadJobTitle, this, Activator.PLUGIN_ID, null) {
			@Override
			protected String getErrorMessage()
			{
				return Messages.get().ScriptLibrary_LoadJobError;
			}

			@Override
			protected void runInternal(IProgressMonitor monitor) throws Exception
			{
				final List<Script> library = session.getScriptLibrary();
				runInUIThread(new Runnable() {
					@Override
					public void run()
					{
						viewer.setInput(library.toArray());
					}
				});
			}
		}.start();
	}
	
	/**
	 * Create new script
	 */
	private void createNewScript()
	{
		final CreateScriptDialog dlg = new CreateScriptDialog(getSite().getShell(), null);
		if (dlg.open() == Window.OK)
		{
			new ConsoleJob(Messages.get().ScriptLibrary_CreateJobTitle, this, Activator.PLUGIN_ID, null) {
				@Override
				protected void runInternal(IProgressMonitor monitor) throws Exception
				{
					final long id = session.modifyScript(0, dlg.getName(), ""); //$NON-NLS-1$
					runInUIThread(new Runnable()
					{
						@Override
						public void run()
						{
							Object[] input = (Object[])viewer.getInput();
							List<Script> list = new ArrayList<Script>(input.length);
							for(Object o : input)
								list.add((Script)o);
							final Script script = new Script(id, dlg.getName(), ""); //$NON-NLS-1$
							list.add(script);
							viewer.setInput(list.toArray());
							viewer.setSelection(new StructuredSelection(script));
							actionEdit.run();
						}
					});
				}

				@Override
				protected String getErrorMessage()
				{
					return Messages.get().ScriptLibrary_CreateJobError;
				}
			}.start();
		}
	}
	
	/**
	 * Edit script
	 */
	private void editScript()
	{
		IStructuredSelection selection = (IStructuredSelection)viewer.getSelection();
		Script script = (Script)selection.getFirstElement();
		try
		{
			getSite().getPage().showView(ScriptEditorView.ID, Long.toString(script.getId()), IWorkbenchPage.VIEW_ACTIVATE);
		}
		catch(PartInitException e)
		{
			MessageDialogHelper.openError(getSite().getWorkbenchWindow().getShell(), Messages.get().ScriptLibrary_Error, String.format(Messages.get().ScriptLibrary_EditScriptError, e.getMessage()));
		}
	}

	/**
	 * Edit script
	 */
	private void renameScript()
	{
		IStructuredSelection selection = (IStructuredSelection)viewer.getSelection();
		final Script script = (Script)selection.getFirstElement();
		final CreateScriptDialog dlg = new CreateScriptDialog(getSite().getShell(), script.getName());
		if (dlg.open() == Window.OK)
		{
			new ConsoleJob(Messages.get().ScriptLibrary_RenameJobTitle, this, Activator.PLUGIN_ID, null)
			{
				@Override
				protected void runInternal(IProgressMonitor monitor) throws Exception
				{
					session.renameScript(script.getId(), dlg.getName());
					runInUIThread(new Runnable()
					{
						@Override
						public void run()
						{
							Object[] input = (Object[])viewer.getInput();
							List<Script> list = new ArrayList<Script>(input.length);
							for(Object o : input)
							{
								if (((Script)o).getId() != script.getId())
									list.add((Script)o);
							}
							final Script newScript = new Script(script.getId(), dlg.getName(), script.getSource());
							list.add(newScript);
							viewer.setInput(list.toArray());
							viewer.setSelection(new StructuredSelection(newScript));
						}
					});
				}
				
				@Override
				protected String getErrorMessage()
				{
					return Messages.get().ScriptLibrary_RenameJobError;
				}
			}.start();
		}
	}

	/**
	 * Delete selected script(s)
	 */
	@SuppressWarnings("rawtypes")
	private void deleteScript()
	{
		if (!MessageDialogHelper.openQuestion(getSite().getShell(), Messages.get().ScriptLibrary_Confirmation, Messages.get().ScriptLibrary_ConfirmationText))
			return;
		
		final IStructuredSelection selection = (IStructuredSelection)viewer.getSelection();
		
		new ConsoleJob(Messages.get().ScriptLibrary_DeleteJobTitle, this, Activator.PLUGIN_ID, null) {
			@Override
			protected void runInternal(IProgressMonitor monitor) throws Exception
			{
				Iterator it = selection.iterator();
				while(it.hasNext())
				{
					Script script = (Script)it.next();
					session.deleteScript(script.getId());
				}
			}

			/* (non-Javadoc)
			 * @see org.netxms.ui.eclipse.jobs.ConsoleJob#jobFinalize()
			 */
			@Override
			protected void jobFinalize()
			{
				refreshScriptList();
			}

			@Override
			protected String getErrorMessage()
			{
				return Messages.get().ScriptLibrary_DeleteJobError;
			}
		}.start();
	}
	
	/**
	 * Copy script name to clipboard
	 */
	private void copyNameToClipboard()
	{
      IStructuredSelection selection = viewer.getStructuredSelection();
	   WidgetHelper.copyToClipboard(((Script)selection.getFirstElement()).getName());
	}
	
   /**
    * Enable or disable filter
    * 
    * @param enable New filter state
    */
   private void enableFilter(boolean enable)
   {
      filterText.setVisible(enable);
      FormData fd = (FormData)viewer.getTable().getLayoutData();
      fd.top = enable ? new FormAttachment(filterText) : new FormAttachment(0, 0);
      content.layout();
      if (enable)
      {
         filterText.setFocus();
      }
      else
      {
         filterText.setText(""); //$NON-NLS-1$
         onFilterModify();
      }
      settings.put("ScriptLibrary.showFilter", enable);
   }

   /**
    * Handler for filter modification
    */
   private void onFilterModify()
   {
      final String text = filterText.getText();
      filter.setFilterString(text);
      viewer.refresh(false);
   }

   /**
    * @param b
    * @param defval
    * @return
    */
   private boolean getBooleanFromSettings(String name, boolean defval)
   {
      String v = settings.get(name);
      return (v != null) ? Boolean.valueOf(v) : defval;
   }
}
